/***********************************************************************
 *  Copyright (c) 2011: 
 *  Istituto Nazionale di Fisica Nucleare (INFN), Italy
 *  Consorzio COMETA (COMETA), Italy
 * 
 *  See http://www.infn.it and and http://www.consorzio-cometa.it for details on
 *  the copyright holders.
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *  http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 ***********************************************************************/

package it.infn.ct.security.actions;

import com.opensymphony.xwork2.ActionSupport;
import it.infn.ct.security.entities.UserRequest;
import it.infn.ct.security.utilities.LDAPUtils;
import it.infn.ct.security.utilities.Organization;
import java.util.LinkedHashMap;
import java.util.Map;
import org.apache.struts2.ServletActionContext;
import org.hibernate.Session;
import org.hibernate.SessionFactory;

/**
 *
 * @author Marco Fargetta <marco.fargetta@ct.infn.it>
 */
public class ValidateUser extends ActionSupport{
    private String id;
    private UserRequest userReq;
    private Map<String, String> orgs;

    @Override
    public String execute() throws Exception {
        
        SessionFactory factory = (SessionFactory) ServletActionContext.getServletContext().getAttribute("IDPPublic.hibernatefactory");
        Session session = factory.openSession();
        userReq = (UserRequest) session.get(UserRequest.class, Long.parseLong(id));
        session.close();
        
        userReq.setOrganizationDN(LDAPUtils.getOrgDN(userReq.getOrganization(), userReq.getCountry()));
        orgs= new LinkedHashMap<String, String>();
        
        for(Organization o: LDAPUtils.getOrgList(userReq.getCountry())){
            orgs.put(o.getDn(), o.getKey()+" - "+o.getDescription());
        }
        
        return SUCCESS;
    }

    public UserRequest getUserReq() {
        return userReq;
    }

    public void setUserReq(UserRequest userReq) {
        this.userReq = userReq;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public Map<String, String> getOrgs() {
        return orgs;
    }

    public void setOrgs(Map<String, String> orgs) {
        this.orgs = orgs;
    }


}
